/*
 * Memory and machine-specific definitions.  Used in C and assembler.
 */

/*
 * Sizes
 */
#define	BI2BY		8					/* bits per byte */
#define	BI2WD		32					/* bits per word */
#define	BY2WD		4					/* bytes per word */
#define BY2V		8					/* bytes per vlong */
#define	BY2PG		4096				/* bytes per page */
#define	WD2PG		(BY2PG/BY2WD)		/* words per page */
#define	PGSHIFT		12					/* log2(BY2PG) */
#define ROUND(s, sz)	(((s)+((sz)-1))&~((sz)-1))
#define PGROUND(s)	ROUND(s, BY2PG)
#define BLOCKALIGN	8

#define	MAXMACH		1				/* max # cpus system can run */
#define	KSTACK		4096			/* Size of kernel stack */

/*
 * Magic registers
 */
#define	USER	24	/* R24 is up-> */
#define	MACH	25	/* R25 is m-> */


/*
 * Time
 */
#define	HZ		100			/* clock frequency */
#define	MS2HZ		(1000/HZ)
#define	TK2SEC(t)	((t)/HZ)		/* ticks to seconds */


/* Coprocessor 0 registers - p. */

#define TLBINDEX		0
#define TLBRAND			1
#define TLBENTRYLO0		2
#define TLBENTRYLO1		3
#define TLBCONTEXT		4
#define TLBPAGEMASK		5
#define TLBWIRED		6
#define BADVADDR		8
#define COUNT			9
#define TLBENTRYHI		10
#define COMPARE			11
#define STATUS			12
#define CAUSE			13
#define EPC				14
#define PRID			15
#define CONFIG			16
/* #define CONFIG1		16 */
#define WATCHLO			18
#define WATCHHI			19
#define XCONTEXT		20
#define DEBUG			23
#define DEPC			24
#define PERFCNT			25
#define ERRCTL			26
#define CACHEERR		27
#define CACHETAGLO		28
#define CACHEDATALO		28
#define CACHETAGHI		29
#define CACHEDATAHI		29
#define ERROREPC		30
#define DESAVE			31

/* Status Register - p.91-97 */
#define CPSUSABLE	0x30000000
#define CP0			0x10000000
#define CP1			0x20000000
#define RP			0x08000000
#define FR			0x04000000
#define RE			0x02000000
#define MX			0x01000000
#define PX			0x00800000
#define BEV			0x00400000
#define TS			0x00200000
#define SR			0x00100000
#define NMI			0x00080000
/* Interrupt masking #define 0x00000000 */
#define KX			0x00000080
#define SX			0x00000040
#define UX			0x00000020
#define KSU			0x00000018
#define KKERNEL		0x00000000
#define KSUPER		0x00000010
#define KUSER		0x00000008
#define ERL			0x00000004
#define EXL			0x00000002
#define IE			0x00000001

/* Cause Register - p.105-108 */
#define BD			0x80000000
#define TI			0x40000000
#define ECP1		0x20000000
#define ECP0		0x10000000
#define DC			0x08000000
#define PCI			0x04000000
#define IV			0x00800000
#define WP			0x00400000
#define HARDIP5		0x00008000
#define HARDIP4		0x00004000
#define HARDIP3		0x00002000
#define HARDIP2		0x00001000
#define HARDIP1		0x00000800
#define HARDIP0		0000000400
#define SOFTIP1		0x00000200
#define SOFTIP0		0x00000100

#define	USEG		0x0000000000000000
#define KSEG0		0xffffffff80000000
#define KSEG1		0xffffffffa0000000
//#define SSEG		0xffffffffc0000000
#define KSEG3		0xffffffffe0000000

#define XUSEG		0x0000000000000000 /* must have KX = 1 */
#define XSSEG		0x4000000000000000
#define XKPHYS		0x8000000000000000
#define XKSEG		0xc000000000000000 /* must have UX = 1 */


#define	KZERO		KSEG0				/* base of kernel address space */
#define	KTZERO		(KZERO+0x20000)		/* first address in kernel text */
#define	MACHADDR	(KTZERO-MAXMACH*BY2PG)	/* warning: rdbg is near here */

/* carrera */
#define	PIDXSHFT	12
#define	PIDX		(0x7<<PIDXSHFT)
#define	KMAPADDR	0xE4000000
#define	KMAPMASK	0xFF000000
#define	KMAPSHIFT	15
#define	NCOLOR		8
#define	getpgcolor(a)	(((ulong)(a)>>PIDXSHFT)&7)

#define	PTEGLOBL	(1<<0)
#define	PTEVALID	(1<<1)
#define	PTEWRITE	(1<<2)
#define	PTERONLY	0
#define	PTEALGMASK	(7<<3)
#define	PTEUNCACHED	(2<<3)
#define	PTENONCOHER	(3<<3)
#define	PTECOHERXCL	(4<<3)
#define	PTECOHERXCLW	(5<<3)
#define	PTECOHERUPDW	(6<<3)
#define	IOPTE		(PTEGLOBL|PTEVALID|PTEWRITE|PTEUNCACHED)

#define	PTEPID(n)	(n)
#define	TLBPID(n)	((n)&0xFF)
#define	PTEMAPMEM	(1024*1024)	
#define	PTEPERTAB	(PTEMAPMEM/BY2PG)
#define	STLBLOG		13
#define	STLBSIZE	(1<<STLBLOG)
#define	KPTELOG		7
#define	KPTESIZE	(1<<KPTELOG)
#define	SEGMAPSIZE	512
#define SSEGMAPSIZE	16

#define	NTLBPID	256	/* number of pids */
#define	NTLB	48	/* number of entries */
#define	TLBROFF	5	/* offset of first randomly indexed entry */


#define SP			R29

